<?php
/**
 * Nic_Device_SNMP:: Nic_Device implementation to retrieve data from
 *                        a SNMP service.
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @version $Revision: 1.4 $
 * @since   Nic 0.2
 * @package Nic
 */
class Nic_Device_SNMP extends Nic_Device {

    /* Store Conenctions Paramters */
    var $params;

    /**
     * Render the current details of the device in an HTML table
     *
     * @param string    $templagePath  Path to the templates to use.
     * @param string    $deviceid      ID of the device, if set is use to create links
     *
     */
    function showDetails($templatePath, $device_id)
    {
        require $templatePath . '/header.inc';

        $details = $this->retrieveStoreableData();

        foreach ($details as $description => $values) {
            $value = $values['value'];
            require $templatePath . '/row.inc';
        }

        foreach ($this->_graphs as $graph_id => $graph) {
            require $templatePath . '/row-graph.inc';
        }

        require $templatePath . '/footer.inc';
    }

    /**
     * Retrieve the current details of the device.
     *
     * @return    array      KEY: description of detail VALUE: value of that details
     */
    function retrieveDetails()
    {
        $details = array();
        if (!extension_loaded('snmp')) {
            return $details;
        }

        foreach ($this->oids as $oid) {
            $details[$oid['name']] = array();
            $details[$oid['name']]['value'] = @snmpget($this->params['hostspec'] . ':' . $this->params['port'], $this->params['community'], $oid['oid'], $this->params['timeout'], 2);

            $details[$oid['name']]['value'] = $this->_cleanResponce($responce);

            if (!array_key_exists('data_type', $oid)) {
                $details[$oid['name']]['data_type'] = NIC_DATA_COUNTER;
            } else {
                $details[$oid['name']]['data_type'] = $oid['data_type'];
            }
        }

        return $details;
    }

    /**
     * Retrieve the current details of the device that can be stored
     * in a storage backend.
     *
     * @return    array      KEY: description of detail VALUE: value of that details
     *
     */
    function retrieveStoreableData()
    {
        $details = array();
        if (!extension_loaded('snmp')) {
            return $details;
        }

        foreach ($this->oids as $oid) {
            if ($oid['storeable']) {
                $details[$oid['name']] = array();
                $responce = @snmpget($this->params['hostspec'] . ':' . $this->params['port'], $this->params['community'], $oid['oid'], $this->params['timeout'], 2);
                $details[$oid['name']]['value'] = $this->_cleanResponce($responce);

                if (!array_key_exists('data_type', $oid)) {
                    $details[$oid['name']]['data_type'] = NIC_DATA_COUNTER;
                } else {
                    $details[$oid['name']]['data_type'] = $oid['data_type'];
                }
            }
        }

        return $details;
    }

    /**
     * Clean the responce from an snmp get as differenct versions return different things.
     */
    function _cleanResponce($responce)
    {
        $pos = strpos($responce, 'Counter32:');
        if ($pos !== false) {
            return trim(substr($responce, $pos + 10));
        }
        $pos = strpos($responce, 'Gauge32');
        if ($pos !== false) {
            return trim(substr($responce, $pos + 8));
        }
        $pos = strpos($responce, 'Timeticks:');
        if ($pos !== false) {
            return trim(substr($responce, $pos + 10));
        }
        $pos = strpos($responce, '=');
        if ($pos !== false) {
            return trim(substr($responce, $pos + 1));
        }
        return $responce;
    }

}
