<?php

/* Data Type Constants */

/* @const NIC_DATA_GAUGE
 * A measure of how much of somethings is being used
 * eg percentage of CPU usage.
 */
define('NIC_DATA_GAUGE', 701);

/* @const NIC_DATA_RATE
 * A Rate is the number of times something happens per second.
 * eg queries per second
 */
define('NIC_DATA_RATE',  702);

/* @const NIC_DATA_COUNTER
 * A Counter increases as things happen
 * eg amount of data transfered
 */
define('NIC_DATA_COUNTER', 703);

/* @const NIC_DATA_INTEGER
 * An integer data type is the number things that exist.
 * eg number of active connections or sessions
 */
define('NIC_DATA_INTEGER', 704);

/* Device States Constants */
/** @const NIC_STATUS_NORMAL  Device is responding normally  */ define('NIC_STATUS_NORMAL',  810);
/** @const NIC_STATUS_WARN    Device is in a warning state   */ define('NIC_STATUS_WARN',    820);
/** @const NIC_STATUS_FAULT   Device is in a fault state     */ define('NIC_STATUS_FAULT',   830);
/** @const NIC_STATUS_UNKNOWN Device is in an unknown state  */ define('NIC_STATUS_UNKNOWN', 840);

/**
 * Nic:: Contains commont functions for Nic
 *
 * $Horde: nic/lib/Nic.php,v 1.25 2004/11/20 20:13:17 jan Exp $
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @version $Revision: 1.25 $
 * @since   Nic 0.2
 * @package Nic
 */
class Nic {

    /**
     * Returns the clear text name of the given service.
     *
     * @param string $service  The service name.
     * @return string          The cleartext name.
     */
    function getServiceName($service)
    {
        $services = array(
            'status' => _("Network Status"),
            'finger' => _("Finger"),
            'icq' => _("ICQ"),
            'lookup' => _("IP Lookup"),
            'mailaddrcheck' => _("Verify Address"),
            'ping' => _("Ping"),
            'relay' => _("SMTP"),
            'services' => _("Services"),
            'traceroute' => _("Traceroute"),
            'whois' => _("Whois"),
            'dns' => _("DNS Lookup")
        );
        return $services[$service];
    }

    /**
     * Fetch all storable details from devices and add them to the storage backend.
     */
    function fetchAndStore()
    {
        global $conf, $devices;

        require_once NIC_BASE . '/lib/Device.php';
        require_once NIC_BASE . '/lib/Storage.php';
        require_once NIC_BASE . '/lib/Status.php';

        $status_storage = &Nic_Status::singleton();
        $data_storage   = &Nic_Storage::singleton();

        foreach ($devices as $deviceid => $device) {
            printf(_("Fetching Status for %s on %s"), $deviceid, $device['params']['hostname']);
            $dev = &Nic_Device::singleton($device['driver'], $device['params']);

            $now = time();
            $status = $dev->getStatus();
            $status_storage->setStatus($now, $device['params']['hostname'], $device['params']['subsystem'], $status['status'], $status['message']);
            echo " done\n";

            printf(_("Fetching Data for %s on %s"), $deviceid, $device['params']['hostname']);
            $data = $dev->retrieveStoreableData();
            foreach ($data as $data_item => $data_value) {
                $data_storage->add($now, $device['params']['hostname'], $device['params']['subsystem'], $data_item, $data_value['value']);
            }
            echo " done\n";
        }
    }

    /**
     * Purge old data and statusrecords
     */
    function purge()
    {
        global $conf;

        $keepseconds = $conf['purge']['keepdays'] * 24 * 60 * 60;
        $removeolder = time() - $keepseconds; 
        //Set treshold for removing records
        $treshold    = $removeolder - 86400;

        require_once NIC_BASE . '/lib/Storage.php';
        require_once NIC_BASE . '/lib/Status.php';
                                                                                                                                                             
        $data_storage   = &Nic_Storage::singleton();

        // Only remove if treshold is exceeded
        if ($data_storage->oldest() < $treshold) {
            echo 'Purging records from before '. $removeolder;

            if ($result = $data_storage->purge($removeolder)) {
                $status_storage = &Nic_Status::singleton();
                $result = $status_storage->purgeStatus($removeolder);
            }

            if (!is_a($result, 'PEAR_error')) {
                echo " done\n";
            } else {
                echo " FAILED\n";
            }
        }
    }


    function getStatusHTML($status)
    {
        global $hostname, $subsystem, $registry;

        switch ($status['status']) {
        case NIC_STATUS_NORMAL:
            $color = '#008000';
            $status_text = _("Working Normally");
            break;

        case NIC_STATUS_WARN:
            $color = '#ff8000';
            $status_text = _("Warning");
            break;

        case NIC_STATUS_FAULT:
            $color = '#ff0000';
            $status_text = _("Fault");
            break;

        default:
            return '';
        }

        $img = Horde::url($registry->get('webroot', 'horde') . '/services/images/pixel.php');
        $img = Util::addParameter($img, 'c', $color);

        $url = Util::addParameter('subsystem.php', 'hostname', $hostname);
        $url = Util::addParameter($url, 'subsystem', $subsystem);

        $comment = split("[\n]|[\r]|[\n\r]", rtrim($status['comment']));
        $comment = implode("\n", $comment);

        return Horde::linkTooltip(Horde::applicationUrl($url), $status_text, '', '', '', $comment) . "<img src='$img' style='width:15px;height:15px;border:none'></a>";
    }

    /**
     * Build Nic's list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        global $conf;

        require_once 'Horde/Menu.php';

        $menu = &new Menu();

        /* NIC menu items. */
        foreach ($conf['features'] as $service => $allowed) {
            if ($allowed) {
                $menu->add(Horde::applicationUrl($service . '.php'), Nic::getServiceName($service), $service . '.png');
            }
        }

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }


}
